"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/comment.py $
$Id: comment.py 25468 2004-10-26 20:01:02Z dbinger $

Provides Comment.
"""

from dulcinea.base import DulcineaPersistent
from dulcinea.timestamped import Timestamped
from dulcinea.typeutils import typecheck
from dulcinea.user import DulcineaUser

class Comment(DulcineaPersistent, Timestamped):
    """A single comment with a bit of audit trail: its author and the
    time it was made.

    Instance attributes:
      text : string
        Text of comment
      user : DulcineaUser
        User who wrote the comment

    All of these attributes should be treated as read-only after the
    Comment instance has been created.
    """

    text_is = str
    user_is = DulcineaUser

    def __init__ (self, text, user):
        Timestamped.__init__(self)
        typecheck(user, DulcineaUser, allow_none=1)
        self.text = text
        self.user = user

    def get_text(self):
        return self.text

    def get_user(self):
        return self.user

    def __str__ (self):
        return "%s at %s" % (self.user, self.get_timestamp())

    def __cmp__ (self, other):
        return cmp((self.text, self.user, self.get_timestamp()),
                   (other.text, other.user, other.get_timestamp()))




#!/www/python/bin/python
"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/bin/check_durus.py $
$Id: check_durus.py 25796 2004-12-16 17:48:26Z dbinger $
"""
import sys
from sets import Set
from dulcinea.spec import get_spec_problems, mapping, sequence, any, \
     get_specs, nspec, get_spec_doc
from durus.connection import Connection
from durus.file_storage import FileStorage
from durus.client_storage import ClientStorage
from durus.storage import gen_oid_class, get_reference_index
from durus.storage_server  import DEFAULT_PORT, DEFAULT_HOST

class ObjectLocator:

    def __init__(self, connection):
        self.connection = connection
        self.reference_index = get_reference_index(connection.get_storage())

    def where_is(self, obj):
        root = self.connection.get_root()
        def locate(obj, seen):
            if len(seen) > 50:
                return '...'
            if obj is root:
                return 'root'
            oid = obj._p_oid
            for referring_oid in self.reference_index[oid]:
                if referring_oid not in seen:
                    seen.add(referring_oid)
                    break
            else:
                return repr(obj) # give up
            referring = self.connection.get(referring_oid)
            ref = locate(referring, seen)
            for key, value in vars(referring).iteritems():
                if obj is value:
                    return "%s.%s" % (ref, key)
                try:
                    for k, v in value.iteritems():
                        if obj is v:
                            return "%s[%r]" % (ref, k)
                except AttributeError:
                    pass
                try:
                    for k, v in enumerate(value):
                        if obj is v:
                            return "%s[%r]" % (ref, k)
                except TypeError:
                    pass
            return '%s ... ' % ref
        return locate(obj, Set())

def gen_class_name_oids(storage):
    oids_by_class_name = {}
    for oid, class_name in gen_oid_class(storage):
        oids_by_class_name.setdefault(class_name, []).append(oid)
    class_name_oids = oids_by_class_name.items()
    class_name_oids.sort()
    for item in class_name_oids:
        yield item

def format_specs(klass):
    if klass.__bases__:
        bases = '(%s):' % ', '.join(
            [b.__name__ for b in klass.__bases__])
    else:
        bases = ":"
    s = "class %s%s\n    " % (klass.__name__, bases)
    s += get_spec_doc(klass).replace('\n', '\n    ') or 'pass'
    s += "\n\n"
    return s

def check_durus(file, host, port, schema, klasses):
    if file:
        storage = FileStorage(file, readonly=True)
    else:
        storage = ClientStorage(host=host, port=port)
    connection = Connection(storage)
    where_is = ObjectLocator(connection).where_is
    try:
        import grouch
        _unused = grouch # so that import checker does not complain        
        get_specs = Grouchy(connection, klasses).get_specs
    except ImportError:
        pass
    if schema:
        schema_output = open(schema, 'w')
        written = Set()
    num_instances_checked = 0
    for class_name, oids in gen_class_name_oids(storage):
        if klasses and class_name not in klasses:
            continue
        print "%6d %s" % (len(oids), class_name)
        for oid in oids:
            obj = connection.get(oid)
            specs = get_specs(obj)
            if schema and obj.__class__ not in written:
                written.add(obj.__class__)
                schema_output.write(format_specs(obj.__class__))
            num_instances_checked += 1
            problems = get_spec_problems(obj, specs=specs)
            if problems:
                print
                print where_is(obj)
                for problem in problems:
                    for line in problem.splitlines():
                        if len(line) > 200:
                            line = line[:200] + ' ...'
                        print ' ', line
                break
            obj._p_set_status_ghost()
    print num_instances_checked, 'instances checked.'

class Grouchy:

    def __init__(self, connection, klass_names):
        from grouch.type_parser import TypeScanner, TypeParser
        self.scanner = TypeScanner()
        self.parser = TypeParser(schema=self)
        self.known_specs = {}
        # Build a base environment to use for evaluating typestrings.
        env = __builtins__.__dict__.copy()
        classes = [
            connection.get(oids[0]).__class__
            for name, oids in gen_class_name_oids(connection.get_storage())
            if not klass_names or name in klass_names]
        # The following line should go away.
        import durus._persistent; sys.modules['_persistent'] = durus._persistent
        while classes:
            klass = classes.pop()
            env[klass.__name__] = klass
            env["%s.%s" % (klass.__module__, klass.__name__)] = klass
            for value in sys.modules[klass.__module__].__dict__.itervalues():
                if hasattr(value, '__bases__') and not value.__name__ in env:
                    classes.append(value)
            for base in getattr(klass, '__mro__', klass.__bases__):
                if base.__name__ not in env:
                    classes.append(base)
        env.update(sys.modules)
        import string
        _unused = string # so that import checker does not complain
        env['string'] = str
        import dulcinea.spec
        env['boolean'] = dulcinea.spec.boolean
        env['any'] = dulcinea.spec.anything
        from types import ClassType, TypeType
        env['class'] = any(ClassType, TypeType)
        self.env = env
        self.klass_env = env

    def get_specs(self, obj):
        from grouch.schema_util import parse_docstring_helper
        def set_spec(klass, name, typespec):
            is_name = name + '_is'
            if is_name not in klass.__dict__:
                try:
                    setattr(klass, is_name, nspec(typespec)) # allows None.
                except TypeError, e:
                    return False
            return True
        def get_class_specs(klass):
            errors = []
            attrs = []
            if klass in __builtins__.__dict__.itervalues():
                self.known_specs[klass] = {}
                return {}
            if klass in self.known_specs:
                return self.known_specs[klass]
            try:
                env = self.env.copy()
                env.update(sys.modules[klass.__module__].__dict__)
                env.update(klass.__dict__)
                self.klass_env = env
                self.klass_env['string'] = str
                try:
                    if klass.__doc__:
                        errors, attrs = parse_docstring_helper(
                            self, klass.__doc__,
                            klass.__name__)
                except Exception, e:
                    if 'Instance attributes:' not in str(e):
                        e.args += ("in %s" % klass.__name__,)
                        raise
                for name, type in attrs:
                    if not set_spec(klass, name, type):
                        self.known_specs[klass] = dict(attrs)
                        break
                for b in klass.__bases__:
                    base_specs = get_class_specs(b)
                    if base_specs != get_specs(b):
                        for key, val in base_specs.iteritems():
                            if not set_spec(klass, key, val):
                                    break
            except ValueError, exc:
                errors = [str(exc)]

            if errors:
                print >>sys.stderr, '\nWARNING: '.join([''] + errors),

            if klass not in self.known_specs:
                self.known_specs[klass] = get_specs(klass)
            return get_class_specs(klass)
        specs = get_class_specs(obj.__class__)
        return specs

    def parse_type(self, typestr):
        self.current_typestr = typestr
        tokens = self.scanner.tokenize(typestr)
        return self.parser.parse(tokens)

    def get_type(self, s):
        if type(s) is type:
            return s
        if type(s) is str:
            if s in self.klass_env:
                return self.klass_env.get(s)
        try:
            return eval(s, self.klass_env)
        except Exception, e:
            e.args += ("Can't parse: %r in %r" % (s, self.current_typestr),)
            raise

    def make_list_type(self, *args):
        return list(args)

    def make_tuple_type(self, element_types, extended):
        assert not extended
        return tuple(element_types)

    def make_dictionary_type(self, key, value):
        return {key:value}

    def make_union_type(self, typelist):
        return any(*typelist)

    def make_set_type(self, element_type):
        return sequence(element_type, Set)

    def make_instance_container_type(self, name, element_type):
        if type(name) is not str:
            name = name.name # grouch.type_parser.Name
        if type(element_type) is dict:
            container = mapping
        else:
            container = sequence
            element_type = element_type[0]
        if name in self.env:
            return container(element_type, self.env.get(name))
        return container(element_type)

def check_durus_main():
    from optparse import OptionParser
    parser = OptionParser()
    parser.set_description("Searches for spec anomolies in a durus database.")
    parser.add_option(
        '--file', dest="file", default=None,
        help="If this is not given, the storage is through a Durus server.")
    parser.add_option(
        '--port', dest="port", default=DEFAULT_PORT,
        type="int",
        help="Port the server is on. (default=%s)" % DEFAULT_PORT)
    parser.add_option(
        '--host', dest="host", default=DEFAULT_HOST,
        help="Host of the server. (default=%s)" % DEFAULT_HOST)
    parser.add_option(
        '--schema', dest="schema", default=None,
        help="If given, a schema will be written to this file")
    parser.add_option(
        '--class', dest="classes", default=[], action="append",
        help="If given, limit study to this class.")
    (options, args) = parser.parse_args()
    check_durus(options.file, options.host, options.port, options.schema,
                options.classes)

if __name__ == '__main__':
    check_durus_main()




"""$URL: svn+ssh://svn/repos/trunk/dulcinea/lib/gallery.py $
$Id: gallery.py 23659 2004-03-08 22:44:18Z dbinger $
"""

from dulcinea.base import DulcineaPersistent
from dulcinea.attachable import Attachable

class Gallery (DulcineaPersistent, Attachable):
    """
    Instance attributes: none
    """

    def get_allowed_mime_types(self, user=None):
        return ['image/jpeg',
                'image/gif',
                'image/png']




"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/extent.py $
$Id: extent.py 25070 2004-09-10 18:02:07Z dbinger $
"""

from durus.storage import gen_oid_class
from dulcinea import local

class Extent:
    def __init__(self, *class_names):
        self.objects = {}
        if class_names:
            self.build(*class_names)

    def build(self, *class_names):
        for class_name in class_names:
            self.objects[class_name] = []
        connection = local.get_connection()
        storage = connection.get_storage()
        for oid, class_name in gen_oid_class(storage, *class_names):
            self.objects[class_name].append(connection.get(oid))

    def all(self, *class_names):
        for class_name in class_names:
            if class_name not in self.objects:
                raise RuntimeError, 'extent was not built for %r' % class_name
        for class_name in class_names:
            for obj in self.objects[class_name]:
                yield obj



'''$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/ui/publisher.py $
$Id: publisher.py 26284 2005-03-04 22:31:47Z rmasse $
'''

import os
import time
import random
import tempfile

from durus.error import ConflictError
from quixote import get_response, get_request
from quixote.publish import Publisher
from quixote.html import stringify
from quixote.http_response import HTTPResponse, Stream
from dulcinea import local
from dulcinea.site_util import get_root_directory
from dulcinea.ui.errors import format_exception, RespondNow
from dulcinea.ui.user.util import ensure_signed_in

TIDY_IGNORE = [
    'Warning: <table> lacks "summary" attribute',
    'Warning: trimming empty <div>',
    'Warning: trimming empty <option>',
    ]

class DulcineaPublisher (Publisher):

    # times to retry a request that failed due to a conflict error
    CONFLICT_RETRIES = 3

    TIDY_CHECK = False

    def __init__(self, **kwargs):
        Publisher.__init__(self, get_root_directory(),
                           session_manager=local.get_session_manager(),
                           **kwargs)

    def respond_now(self):
        """
        Exit the publishing loop and return the current response.
        """
        raise RespondNow

    def start_request(self):
        local.get_connection().abort()
        Publisher.start_request(self)

    def format_publish_error(self, exc):
        return format_exception(exc)

    def finish_interrupted_request(self, exc):
        response = get_response()
        # This commits changes and formats a response.
        output = Publisher.finish_interrupted_request(self, exc)
        if isinstance(exc, RespondNow):
            # Use the response present at the time of the exception.
            get_request().response = response
            output = response.body
        return output

    def try_publish(self, request):
        """Try to process a single request.  Use the base class method to the
        do actual the work.  If a conflict error occurs retry a number of
        times.
        """
        for i in range(self.CONFLICT_RETRIES + 1):
            try:
                value = Publisher.try_publish(self, request)
                if value is None:
                    return ensure_signed_in()
                return value
            except ConflictError, exc:
                self.log("ConflictError (%s): retrying request" % exc)
                local.get_connection().abort()
                request.response = HTTPResponse() # reset response object
                time.sleep(random.random() * (i+1))
        else:
            raise RuntimeError("too many conflict errors")

    def filter_output(self, request, output):
        if self.TIDY_CHECK:
            ctype = request.response.content_type
            if (output and
                    (not ctype or ctype=="text/html") and
                    not isinstance(output, Stream)):
                self.html_tidy(request, output)
        return Publisher.filter_output(self, request, output)

    def html_tidy(self, request, output):
        """
        Small hack to call html-tidy for every html
        page which is serverd by quixote.

        It is used for checking the html syntax only, the html
        output won't be changed.
        """
        htmlfile = tempfile.NamedTemporaryFile()
        output = stringify(output)
        if isinstance(output, unicode):
            output = output.encode(request.response.charset)
        htmlfile.write(output)
        htmlfile.flush()
        htmlfile.seek(0)
        fp = os.popen("/usr/bin/tidy -q -e '%s' 2>&1" % htmlfile.name, "r")
        errors = []
        for line in fp:
            line = line.strip()
            if line:
                for ign in TIDY_IGNORE:
                    if ign in line:
                        break
                else:
                    errors.append(line)
        if errors:
            self.log("HTML-Tidy: %s" % request.get_path())
            for line in errors[:20]:
                self.log("   " + line)
            if len(errors) > 20:
                self.log("   [...more errors...]")
        else:
            self.log("HTML-Tidy: %s okay" % request.get_path())
        fp.close()
        htmlfile.close()



"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/base.py $
$Id: base.py 25817 2004-12-21 16:27:07Z dbinger $

Provides universal base classes which all classes in the Dulcinea
library should subclass.  Currently there are two such
universal base classes:
    DulcineaPersistent    for classes whose instances should always be
                          first-class, persistent database objects
    DulcineaBase          for classes whose instances may be transient,
                          but can still be persisted if they happen to
                          be attached to another persistent object

"""

import new
from types import ClassType
from durus.persistent import Persistent

# If you're wondering which of DulcineaBase and DulcineaPersistent to use
# for some class, then use DulcineaPersistent: that will make instances of
# that class "first-class" database objects, subject to the Durus persistence
# machinery.

class DulcineaBase:
    """The standard base class for classes whose instances do not need
    to be first-class database objects.  Instances of DulcineaBase can still
    be stored in the database if they happen to be attached to
    other persistent objects, but they don't really participate in
    persistence -- they just come along for the ride.

    Instance attributes: none

    """

    def copy (self):
        klass = self.__class__
        if type(klass) is ClassType:
            newself = new.instance(self.__class__, {})
        elif isinstance(klass, object):
            newself = klass.__new__(klass)
        else:
            raise RuntimeError, "unknown type of class object: %s" % `klass`
        newself.__dict__.update(self.__dict__)
        return newself

    def __copy__ (self):
        return self.copy()


class DulcineaPersistent (Persistent, DulcineaBase):
    """The standard base class for classes whose instances should be
    first-class database objects that participate fully in the
    persistence mechanism.  Instances of DulcineaPersistent still have to be
    referenced by an existing persistent object, i.e. they must
    ultimately connect back to one of the database root objects.

    Instance attributes: none
    """




#!/www/python/bin/python

"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/test/utest_country.py $
$Id: utest_country.py 25054 2004-09-09 18:04:30Z dbinger $

Test for dulcinea.country.
"""

from types import TupleType
from sancho.utest import UTest
from dulcinea.country import get_country_name, get_country_list


class CountryTest (UTest):

    def check_country_lookup (self):
        assert get_country_name('CA') == "Canada"
        assert get_country_name('AU') == "Australia"
        assert get_country_name('TV') == "Tuvalu"
        try:
            get_country_name('XX')
            assert 0
        except KeyError: pass
        try:
            get_country_name(None)
            assert 0
        except KeyError: pass

    def check_country_list (self):
        list = get_country_list()
        assert len(list) > 200
        assert ('FI', 'Finland') in list
        assert ('BR', 'Brazil') in list
        assert 0 not in [type(x) is TupleType and len(x) == 2 for x in list]


if __name__ == "__main__":
    CountryTest()



"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/persistent_session.py $
$Id: persistent_session.py 25483 2004-10-28 19:12:25Z nascheme $

Provides persistent versions of Quixote's session management classes.
"""

from time import time

from durus.persistent_dict import PersistentDict
from durus.persistent_list import PersistentList
from quixote.session import SessionManager as QuixoteSessionManager
from quixote.session import Session as QuixoteSession
from dulcinea.attachable import Attachable
from dulcinea.base import DulcineaPersistent
from dulcinea.user import DulcineaUser
from dulcinea import local

class DulcineaSession (DulcineaPersistent, QuixoteSession, Attachable):
    """
    Instance attributes:
      _form_tokens : PersistentList [string]
        (overrides Session's attribute for persistence)
      user : DulcineaUser
        (overrides Session's attribute for stronger typing)
        the user object used for access control checks.  Usually
        user == actual_user, but when an admin user is acting as some other
        user, user is the person being impersonated, and actual_user is
        the person actually doing the work (ie.  the admin user)
      actual_user : DulcineaUser
        (see user attribute)
    """
    user_is = DulcineaUser # not None
    actual_user_is = DulcineaUser # not None

    def __init__ (self, id):
        QuixoteSession.__init__(self, id)
        Attachable.__init__(self)
        self.user = local.get_user_db().get_null_user()
        self.actual_user = self.user
        self._form_tokens = PersistentList()

    def __str__ (self):
        # override Session.__str__ to take actual_user into account
        if not self.actual_user:
            return "session %s (no user)" % self.id
        elif self.user == self.actual_user:
            return "session %s (user %s)" % (self.id, self.user)
        else:
            return "session %s (user %s acting as %s)" % (self.id,
                                                          self.actual_user,
                                                          self.user)
    def clear_app_state(self):
        """Override to clear any session data when acting as another user
        """
        pass

    def has_info (self):
        if self.actual_user or self.user or self._form_tokens:
            return 1
        else:
            return 0

    def set_actual_user (self, user):
        self.user = self.actual_user = user


class DulcineaSessionManager (DulcineaPersistent, QuixoteSessionManager):
    """
    Instance attributes:
      sessions : PersistentDict { session_id:string : DulcineaSession }
        (same as in SessionManager, just made persistent)
    """

    ACCESS_TIME_RESOLUTION = 900 # in seconds (don't dirty the session on
                                 # every hit)

    def __init__ (self, session_class=DulcineaSession, session_mapping=None):
        if session_mapping is None:
            session_mapping = PersistentDict()
        QuixoteSessionManager.__init__(self,
                                       session_class=session_class,
                                       session_mapping=session_mapping)

    def get_sessions(self):
        return self.sessions.values()

    def sorted_keys (self):
        keys = list(self.sessions.keys())
        keys.sort()
        return keys

    def forget_changes (self, session):
        local.get_connection().abort()

    def commit_changes (self, session):
        local.get_connection().commit()

    def del_sessions (self, ip=None, age=None):
        """Delete a subset of open sessions selected according to the
        keyword arguments.  Default values delete all sessions.  Sessions
        are selected by combining non-None keyword args with "and": eg. if
        'ip' and 'age' are non-None then any session matching *both* the
        supplied IP address and age (hours since session was accessed) is
        deleted.

        'ip' specifies an IP address (dotted-quad string); it must match
        the IP address of a session exactly.  'age' is the session's age
        (since last access) in hours; any session older than 'age' will
        pass the age test.

        The number of sessions deleted is returned.
        """
        deleted = 0
        now = time()
        for s in list(self.sessions.values()):
            if ip is not None and ip != s.get_remote_address():
                continue
            if age is not None and s.get_access_age(_now=now) < age*3600:
                continue

            # passed all the tests: we'll delete this sucker
            deleted += 1
            del self.sessions[s.id]

        return deleted







#!/www/python/bin/python
"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/test/utest_user_database.py $
$Id: utest_user_database.py 26150 2005-02-16 15:00:46Z dbinger $
"""

from sancho.utest import UTest
from dulcinea.contact import ContactDatabase, Contact
from dulcinea.user import DulcineaUser, DulcineaUserDatabase


class Test (UTest):

    def check_user_database(self):
        user1 = DulcineaUser('1')
        user1.password_hash = 'junk'
        user2 = DulcineaUser('2')
        self.db = DulcineaUserDatabase()
        self.db.add_user(user1)
        assert self.db.get_users() == [user1]
        self.db.add_user(user2)
        assert self.db.get_disabled_users() == [user2]
        assert self.db.get_users() == [user1]
        user2.password_hash = 'junk'
        assert self.db.get_users(sort=True) == [user1, user2]
        assert self.db.get_user('1') == user1
        assert self.db['1'] == user1
        try:
            self.db.add_user(user1)
            raise Exception
        except AssertionError: pass
        assert self.db.get_admin() == None
        assert self.db.get_motd() == ''
        self.db.set_motd('The message')
        assert self.db.get_motd() == 'The message'


    def check_contact_database(self):
        self.db = ContactDatabase()
        assert str(self.db.get_admin()) == 'admin'
        user1 = Contact('1')
        user1.set_email('foo@bar.com')
        self.db.add_user(user1)
        assert self.db.get_matching_user('notfound@bar.com') == None
        assert self.db.get_matching_user('foo@bar.com') == user1

if __name__ == "__main__":
    Test()



#!/www/python/bin/python
# $URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/test/utest_interaction.py $
# $Id: utest_interaction.py 25713 2004-12-07 23:26:47Z rmasse $

from sancho.utest import UTest
from dulcinea.interaction import Interactable
from dulcinea.user import DulcineaUser

class TestUser(DulcineaUser, Interactable):

    def __init__(self, id):
        DulcineaUser.__init__(self, id)
        Interactable.__init__(self)

class InteractableTest (UTest):

    def check_interactions(self):
        user = TestUser('guido')
        assert len(user.get_interaction_history()) == 0
        user.record_interaction(user, 'email', 'email sent')
        assert len(user.get_interaction_history()) == 1
        assert user.get_interaction_codes() == user.INTERACTION_CODES

if __name__ == "__main__":
    import os; os.environ['SITE'] = 'vfab'
    InteractableTest()




#!/www/python/bin/python
"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/ui/test/utest_crumbs.py $
$Id: utest_crumbs.py 26245 2005-02-25 14:45:59Z dbinger $
"""
from quixote import enable_ptl; enable_ptl()

from dulcinea.ui.crumbs import get_crumb_tree, format_crumbs
from dulcinea.ui.crumbs import get_path_directory_list, get_exports
from dulcinea.ui.directory import DynamicExportingDirectory
from quixote.directory import Directory
from quixote.http_request import HTTPRequest
from quixote.logger import DefaultLogger
from quixote.publish import Publisher
from sancho.utest import UTest
import dulcinea.ui.crumbs
import quixote.publish
import sys


class PrintLogger(DefaultLogger):

    def __init__(self):
        self.error_log = sys.stdout
        self.access_log = None
        self.error_email = None

class CrumbTest (UTest):

    def _pre(self):
        quixote.publish._publisher = None

    def _post(self):
        quixote.publish._publisher = None

    def test_get_path_directory_list(self):
        class Dir(DynamicExportingDirectory):
            def _q_traverse(self, component):
                return get_path_directory_list()
        publisher = Publisher(Dir(), logger=PrintLogger())
        publisher._set_request(HTTPRequest(
            None, dict(SCRIPT_NAME='',
                       PATH_INFO='/')))
        assert get_path_directory_list() == []
        directory = Dir()
        assert directory._q_traverse('') == [('./', directory)]

    def test_get_exports1(self):
        exports = [('', '_q_index', 'crumb', 'title')]
        class Dir(DynamicExportingDirectory):
            def get_exports(self):
                return exports
        assert get_exports(Dir()) == exports

    def test_get_exports_fallback(self):
        class Dir(Directory):
            _q_exports = ['', ('a','b'), 'foo']
        print get_exports(Dir())
        assert get_exports(Dir()) == [('', '_q_index', None, ''),
                                      ('a', 'b', '', ''),
                                      ('foo', 'foo', '', '')]

    def test_get_crumb_tree(self):
        class Dir3(DynamicExportingDirectory):
            def get_exports(self):
                return [('', '_q_index', 'crumb3', 'title3'),
                        ('c3', 'n3', 'acrumb', 'atitle')]

            def _q_traverse(self, components):
                return get_crumb_tree()
            n3 = 'ok'
        class Dir2(DynamicExportingDirectory):
            def get_exports(self):
                return [('', '_q_index', 'crumb2', 'title2'),
                        ('c2', 'n2', 'acrumb', 'atitle')]
            n2 = Dir3()
        class Dir1(DynamicExportingDirectory):
            def get_exports(self):
                return [('', '_q_index', 'crumb', 'title'),
                        ('c1', 'n1', 'acrumb', 'atitle')]
            n1 = Dir2()

        directory = Dir1()
        publisher = Publisher(directory, logger=PrintLogger())
        publisher._set_request(HTTPRequest(
            None, dict(SCRIPT_NAME='',
                       PATH_INFO='/')))
        print directory._q_traverse(['c1', 'c2', ''])
        assert directory._q_traverse(['c1', 'c2', ''])  == [
            [('./../../', 'crumb', 'title'),
             ('./../../c1', 'acrumb', 'atitle')],
            [('./../', 'crumb2', 'title2'),
             ('./../c2', 'acrumb', 'atitle')]]

    def test_get_crumb_tree_no_menu(self):
        class Dir2(DynamicExportingDirectory):
            def get_exports(self):
                return [('', '_q_index', 'crumb3', 'title3'),
                        ('c2', 'n2', 'acrumb', 'atitle')]

            def _q_traverse(self, components):
                return get_crumb_tree()
            n2 = 'ok'
        class Dir(DynamicExportingDirectory):
            def get_exports(self):
                return [('', 'c', '', 'title'),
                        ('c1', 'n1', 'no see crumb', 'no see title')]
            n1 = Dir2()

        directory = Dir()
        publisher = Publisher(directory, logger=PrintLogger())
        publisher._set_request(HTTPRequest(
            None, dict(SCRIPT_NAME='',
                       PATH_INFO='/')))
        print directory._q_traverse(['c1', 'c2'])
        assert directory._q_traverse(['c1', 'c2'])  == [
            [('./', 'crumb3', 'title3'), ('./c2', 'acrumb', 'atitle')]]

    def test_format_crumbs(self):
        class Dir3(DynamicExportingDirectory):
            def get_exports(self):
                return [('', '_q_index', 'crumb', 'title'),
                        ('c3', 'n3', 'acrumb', 'atitle')]

            def _q_traverse(self, components):
                return format_crumbs()
            n3 = 'ok'
        class Dir2(DynamicExportingDirectory):
            def get_exports(self):
                return [('', '_q_index', 'crumb', 'title'),
                        ('c2', 'n2', 'acrumb', 'atitle')]
            n2 = Dir3()
        class Dir1(DynamicExportingDirectory):
            def get_exports(self):
                return [('', '_q_index', 'crumb', 'title'),
                        ('c1', 'n1', 'acrumb', 'atitle')]
            n1 = Dir2()

        directory = Dir1()
        publisher = Publisher(directory, logger=PrintLogger())
        publisher._set_request(HTTPRequest(
            None, dict(SCRIPT_NAME='',
                       PATH_INFO='/')))
        crumb_menus = False
        def get_config_value(name):
            return crumb_menus
        dulcinea.ui.crumbs.get_config_value = get_config_value
        print repr(str(directory._q_traverse(['c1', 'c2', ''])))
        crumb_menus = True
        print repr(str(directory._q_traverse(['c1', 'c2', ''])))

if __name__ == "__main__":
    CrumbTest()



"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/keyed.py $
$Id: keyed.py 23930 2004-04-07 19:37:47Z dbinger $
"""

from durus.persistent_dict import PersistentDict
from dulcinea.spec import require, get_spec_problems, anything, mapping

class Keyed:

    """
    An item with an int key used as a key in an KeyedMap.
    Note that the key attribute is set when the item is put in the KeyedMap,
    so there is no set_key() method.

    Instance attributes:
      key : int
    """
    key_is = int

    def __init__(self):
        self.key = None

    def get_key(self):
        return self.key


class KeyedMap:
    """
    A mixin for a simple database that stores a map objects by key.

    Instance attributes:
        mapping : PersistentDict { int: Keyed }
        value_spec : any
            Specifies the type of values allowed in mapping
        key_counter : int
            Next available key number.
    """

    mapping_is = mapping({int:Keyed}, PersistentDict)
    value_spec_is = anything
    key_counter_is = int

    def __init__(self, value_spec=Keyed):
        self.key_counter = 1
        self.value_spec = value_spec
        self.mapping = PersistentDict()

    def get(self, key, *args):
        return self.mapping.get(key, *args)

    def get_mapping(self):
        return self.mapping

    def add(self, value):
        require(value, self.value_spec)
        assert value.key is None
        value.key = self.key_counter
        if get_spec_problems(value):
            value.key = None
            raise TypeError(''.join(get_spec_problems(value)))
        self.key_counter += 1
        self.mapping[value.key] = value




"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/links.py $
$Id: links.py 26260 2005-02-28 15:55:47Z dbinger $
"""
import random
from datetime import datetime
from quixote.html import htmlescape, htmltext, href
from dulcinea.attachable import Attachable
from dulcinea.category import Category, Categorized, CategoryDatabase
from dulcinea.item import Item, ItemFolder
from dulcinea.spec import any, boolean
from dulcinea.typeutils import typecheck
from dulcinea.util import format_text, wrap_paragraphs, sanitize_url

class LinkItem(Item, Attachable, Categorized):
    """
    Instance attributes:
      link_url : string
        Link to relevant target URL
      text : string
        A few sentences describing the link.
      email : string
        Submitter's e-mail address
      first_class : boolean
        Flag to indicate that this link can be shown in a prominent
        place like the home page of a site
    """

    link_url_is = any(str, None)
    text_is = any(str, None)
    email_is = any(str, None)
    first_class = boolean

    def __init__(self):
        Item.__init__(self)
        Attachable.__init__(self)
        Categorized.__init__(self)
        self.link_url = None
        self.text = None
        self.email = None
        self.first_class = False

    def get_allowed_mime_types(self, user=None):
         """() -> [string]
         """
         if user and user.is_admin():
             return Attachable.get_allowed_mime_types(self)
         else:
             return ["image/gif", "image/jpeg", "image/png",
                     "image/tiff", "image/x-ms-bmp"]


    def set_link_url(self, url):
        typecheck(url, str)
        self.link_url = sanitize_url(url)
        self.timestamp = datetime.now()

    def get_link_url(self):
        "Return the URL to the link this item refers to"
        return self.link_url

    def set_text(self, text):
        self.text = text

    def get_text(self):
        return self.text

    def set_email(self, email):
        self.email = email

    def get_email(self):
        return self.email

    def is_first_class(self):
        return self.first_class

    def set_first_class(self, first_class):
        typecheck(first_class, bool)
        self.first_class = first_class

    def get_local_url(self):
        "Return the URL to this Item"
        return "/links/%s/" % self.get_key()

    def img_href(self, width=None):
        files = self.get_attached_files()
        if files:
            return href("%sgoto" % self.get_local_url(), htmltext(
                '<img src="%sfile/%s" %s class="sponsor" alt="%s" />' % (
                self.get_local_url(), files[0].get_id(),
                width and 'width="%s"' % width or '', self.get_title())))
        return htmltext('')

    def as_html(self, full=0, action_links=None):
        title = href("%sgoto" % self.get_local_url(),
                     htmlescape(self.get_title()))
        if not self.approved:
            title += ' ' + href("%sedit" % self.get_local_url(), "(Unapproved)")
        if action_links:
            title += ' ' + action_links
        body = (self.get_text() and format_text(self.get_text())) or ''
        body = self.img_href() + body
        if full:
            email = htmltext('<p><b>Contact email:</b> %s</p>' % (
                href('mailto:%s' % self.get_email(), self.get_email())))
            key = htmltext('<p><b>Key:</b> %s</p>' % self.get_key())
            categories = htmltext('<p><b>Categorys:</b> %s</p> ' % ', '.join(
                [category.get_label() for category in self.get_categories()]))
            url = htmltext('<p><b>URL:</b> %s</p>' % href(
                self.get_link_url(), self.get_link_url()))
            first_class = htmltext('<p><b>First Class</b> %s</p>' % (
                self.is_first_class()))
            body += key + categories + email + url + first_class
        value = htmltext('\n<dt>%s</dt>\n' % title)
        if body:
            value += htmltext('<dd>%s</dd>\n' % body)
        return value

    def as_text(self):
        return "%s\n\n%s\nURL: %s" % (self.get_title(), self.get_text(),
                                      self.get_link_url())
    def as_email(self):
        body = "%s\n" % self.get_title()
        if self.get_link_url():
            body += "(%s)\n" % self.get_link_url()
        body += "\n%s" % wrap_paragraphs(self.get_text())
        return self.title, body

    def as_rss(self, servername):
        return """<item>
        <title>%s<title>
        <link>http://%s%s/goto</link>
        <description>%s</description>
        </item>""" % (htmlescape(self.get_title()),
                      servername,
                      htmlescape(self.get_local_url()),
                      htmlescape(self.get_text()))


class LinkFolder(ItemFolder, CategoryDatabase):
    """
    Instance attributes: none
    
    To add a new leaf Category, create the Category instance, set it's 'name'
    and 'label', choose a parent Category from the LinkFolder.categories
    dictionary and call it's 'add_child' to add the new category, finally call
    LinkFolder.add_category to add the new Category to categories
    """

    def __init__(self):
        ItemFolder.__init__(self)
        CategoryDatabase.__init__(self)
        links_category = Category(name='all')
        links_category.set_label('All Links')
        self.add_category(links_category)

    def links_with_category(self, category):
        return [link for link in self.get_items() if link.in_category(category)]

    def get_all_links(self):
        return self.get_items()

    def get_first_class_links(self):
        return [link for link in self.get_items() if link.is_first_class()]

    def random_first_class_link(self):
        first_class_links = self.get_first_class_links()
        if first_class_links:
            return random.choice(first_class_links)
        return None

def sort_reverse_by_date(link1, link2):
    # Sort links cronologically with most recent first
    return cmp(link2.get_timestamp(), link1.get_timestamp())

def sort_by_title(link1, link2):
    title1 = (link1.get_title() and link1.get_title().lower()) or None
    title2 = (link2.get_title() and link2.get_title().lower()) or None
    return cmp(title1, title2)



"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/interaction.py $
$Id: interaction.py 25864 2005-01-17 15:31:22Z dbinger $
"""

from dulcinea.history import History

class Interactable:

    """Storage for a set of interaction events on an object of interest

    Instance attributes:
      interactions : History
        the interaction history with this interactible object
    """

    interactions_is = History

    EMAIL = 'email'
    MAIL = 'mail'
    PHONE = 'phone'
    FAX = 'fax'
    OTHER = 'other'

    INTERACTION_CODES = (EMAIL, MAIL, PHONE, FAX, OTHER)

    decorator = str

    def __init__(self):
        self.interactions = History()

    def get_decorator(self):
        return self.__class__.decorator

    def with_decorator(self, decorator):
        self.__class__.decorator = decorator
        return self

    def get_interaction_history(self):
        return self.interactions

    def get_interaction_codes(self):
        return self.INTERACTION_CODES

    def record_interaction(self, user, code, comment):
        self.interactions.add_event(user, code, comment)





#!/www/python/bin/python
"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/ui/test/utest_util.py $
$Id: utest_util.py 25520 2004-11-04 15:51:22Z dbinger $
"""
from quixote import enable_ptl
enable_ptl()
from sancho.utest import UTest
import dulcinea.ui.util
from dulcinea.ui.util import csv
from quixote.http_response import HTTPResponse
class MiscTest (UTest):

    def check_csv (self):
        fake_response = HTTPResponse()
        dulcinea.ui.util.get_response = lambda: fake_response
        s = csv(['a','b'], [{'a':1}, {'a':2, 'b':3}])
        assert s == 'a,b\r\n1,\r\n2,3\r\n'
        assert fake_response.content_type == 'text/comma-separated-values'


if __name__ == "__main__":
    MiscTest()



"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/address.py $
$Id: address.py 25248 2004-10-01 20:26:51Z dbinger $

Address-related classes, functions, and constants.
"""
import re
from dulcinea.base import DulcineaPersistent
from dulcinea.country import get_country_name
from dulcinea.typeutils import typecheck

class Address (DulcineaPersistent):
    """
    Instance attributes:
      street1 : string
        1st line of the street address
      street2 : string
        2nd line of the street address
      city : string
      state : string
      zip : string
      country_code : string
        ISO-3166 two-letter country code (see dulcinea.country for
        tools to map between country code and country name)
    """

    STATES = [
        "AL", "AK", "AR", "AZ", "CA", "CO", "CT", "DE", "FL", "GA",
        "HI", "IA", "ID", "IL", "IN", "KS", "KY", "LA", "MA", "MD",
        "ME", "MI", "MN", "MO", "MS", "MT", "NC", "ND", "NE", "NH",
        "NJ", "NM", "NV", "NY", "OH", "OK", "OR", "PA", "RI", "SC",
        "SD", "TN", "TX", "UT", "VA", "VT", "WA", "WI", "WV", "WY",

        # Commonwealths & territories
        "AS", "DC", "FM", "GU", "MP", "PR", "VI",

        # Armed forces codes
        "AA", "AE", "AP"]


    def __init__(self, street1=None, street2=None, city=None,
                 state=None, zip=None, country_code=None):
        """(street1:str=None, street2:str=None, city:str=None, state:str=None,
            zip:str=None, country_code:str=None)
        """
        self._set_street(street1, street2)
        self._set_city(city)
        self._set_state(state)
        self._set_zip(zip)
        self._set_country_code(country_code)

    def __eq__(self, other):
        return (isinstance(other, self.__class__) and
                other.__dict__ == self.__dict__)

    def __ne__(self, other):
        return not self.__eq__(other)

    def is_valid(self):
        """() -> bool

        Return true if this address is valid: either it's empty or
        it's complete (and correct!).
        """
        return self._check_complete() == []

    def _check_complete(self):
        """() -> [(attrname:str, errtype:str)]

        Check whether this address is complete: must have at least one
        line of street address, city, and country; if country is US,
        must have a valid state and ZIP code too.

        Returns a list of (attrname, errtype) 2-tuples explaining any
        problems.  'errtype' is either "missing" or "bad".
        """
        errors = []
        for field in ['street1', 'city', 'country_code']:
            if not getattr(self, field):
                errors.append((field, "missing"))

        # US-specific checks
        if self.country_code == 'US':
            if not self.state:
                errors.append(('state', "missing"))
            else:
                state = self.state.upper()
                if state not in self.STATES:
                    errors.append(('state', "bad"))
                elif state != self.state:
                    self.state = state  # make uppercase

            # Check the Zip code (5 digits, or 5+4 format)
            if not self.zip:
                errors.append(('zip', "missing"))
            elif not self.has_valid_zip():
                errors.append(('zip', "bad"))

        return errors

    def has_valid_zip(self):
        if self.country_code == 'US':
            if not self.zip:
                return False
            return re.match(r'\d{5}(-\d{4})?$', self.zip) is not None
        return True

    def has_valid_state(self):
        if self.country_code == 'US':
            return self.state and self.state in self.STATES
        return True

    def error_message(self):
        msg = ""
        for errors in self._check_complete():
            msg = msg + "%s is %s, " % (
                " ".join([word.capitalize() for word in errors[0].split("_")]),
                errors[1])
        return (msg and msg[:-2] + '.') or msg

    def _set_street(self, street1, street2):
        """(street1:str, street2:str)
        """
        typecheck(street1, str, allow_none=1)
        typecheck(street2, str, allow_none=1)
        self.street1 = street1
        self.street2 = street2

    def _set_city(self, city):
        """(city:str)
        """
        typecheck(city, str, allow_none=1)
        self.city = city

    def _set_state(self, state):
        """(state:str)
        """
        typecheck(state, str, allow_none=1)
        self.state = state

    def _set_country_code(self, country_code):
        """(country_code:str)
        """
        if country_code:
            typecheck(country_code, str)
            country_code = country_code.upper()
            get_country_name(country_code) # raises KeyError if not found
        self.country_code = country_code

    def _set_zip(self, zip):
        """(zip:str)
        """
        typecheck(zip, str, allow_none=1)
        self.zip = zip

    def format(self):
        address = ""
        if self.street1:
            address += self.street1 + "\n"
        if self.street2:
            address += self.street2 + "\n"
        if self.city:
            address += self.city + ", "
        if self.state:
            address += self.state + " "
        if self.zip:
            address += self.zip
        if address[-1:] != "\n":
            address += "\n"
        if self.country_code:
            address += self.country_code
        return address

    def get_street1(self):
        return self.street1

    def get_street2(self):
        return self.street2

    def format_street(self):
        result = self.street1 or ''
        if self.street2:
            result += ', ' + self.street2
        return result

    def get_city(self):
        return self.city

    def get_state(self):
        return self.state

    def get_zip(self):
        return self.zip

    def get_country_code(self):
        return self.country_code

class Addressable:
    """
    Mixin for objects that contain addresses

    Instance attributes:
      address : Address
    """

    address_is = Address

    def __init__(self, address=None, **kwargs):
        self.address = address or Address(**kwargs)

    def set_address(self, address):
        """(address : Address)
        """
        typecheck(address, Address)
        self.address = address

    def get_address(self):
        """() -> Address
        """
        return self.address


class ContactAddress(DulcineaPersistent, Addressable):
    """
    This is an address with the additional information normally needed
    for shipping.

    Instance attributes:
      contact_name : string
      contact_phone_number : string
      company_name : string
    """

    def __init__(self, contact_name=None, company_name=None,
                 contact_phone_number=None, **kwargs):
        Addressable.__init__(self, **kwargs)
        self._set_contact_name(contact_name)
        self._set_contact_phone_number(contact_phone_number)
        self._set_company_name(company_name)

    def __eq__(self, other):
        return (isinstance(other, self.__class__) and
                other.__dict__ == self.__dict__)

    def __ne__(self, other):
        return not self.__eq__(other)

    def _set_contact_name(self, name):
        typecheck(name, str, allow_none=1)
        self.contact_name = name

    def _set_contact_phone_number(self, contact_phone_number):
        typecheck(contact_phone_number, str, allow_none=1)
        self.contact_phone_number = contact_phone_number

    def _set_company_name(self, company_name):
        typecheck(company_name, str, allow_none=1)
        self.company_name = company_name

    def is_valid(self):
        return self._check_complete() == []

    def error_message(self):
        return self.address.error_message()

    def get_contact_name(self):
        return self.contact_name

    def get_contact_phone_number(self):
        return self.contact_phone_number

    def set_address(self, address):
        raise RuntimeError("ContactAddress is treated as immutable")

    def get_company_name(self):
        return self.company_name

    def get_street1(self):
        return self.address.get_street1()

    def get_street2(self):
        return self.address.get_street2()

    def get_city(self):
        return self.address.get_city()

    def get_state(self):
        return self.address.get_state()

    def get_zip(self):
        return self.address.get_zip()

    def get_country_code(self):
        return self.address.get_country_code()

    def has_valid_phone_number(self):
        if not self.contact_phone_number:
            return False
        number_of_digits = 0
        for token in self.contact_phone_number:
            if token in '()-.+EeXxTt ':
                continue
            if not token.isdigit():
                return False
            else:
                number_of_digits += 1
        return number_of_digits >= 10

    def _check_complete(self):
        """() -> [(attrname:str, errtype:str)]

        Returns a list of (attrname, errtype) 2-tuples explaining any
        problems.  'errtype' is either "missing" or "bad".
        """
        errors = self.address._check_complete()
        for field in ['contact_name', 'contact_phone_number']:
            if not getattr(self, field):
                errors.append((field, "missing"))
        if self.contact_phone_number and not self.has_valid_phone_number():
            errors.append(('contact_phone_number', 'bad'))
        return errors

    def format(self):
        if self.get_contact_name():
            output = self.get_contact_name() + "\n"
        else:
            output = ""
        if self.company_name:
            output += self.company_name + "\n"
        output += self.address.format()
        return output


class ContactAddressable:
    """
    Mixin for objects that have a contact address.

    Instance attributes:
      contact_address : ContactAddress
    """

    contact_address_is = ContactAddress

    def __init__(self):
        self.contact_address = ContactAddress()

    def set_contact_address(self, address):
        """(address : ContactAddress)
        """
        typecheck(address, ContactAddress)
        self.contact_address = address

    def get_contact_address(self):
        """() -> ContactAddress
        """
        return self.contact_address






#!/www/python/bin/python
"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/bin/unknown.py $
$Id: unknown.py 23142 2003-11-11 21:18:02Z dbinger $
"""

from dulcinea.code_util import report_unknown_names, main

main(report_unknown_names)



#!/www/python/bin/python
"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/test/utest_timestamped.py $
$Id: utest_timestamped.py 25452 2004-10-26 17:23:37Z dbinger $
"""
"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/test/utest_timestamped.py $
$Id: utest_timestamped.py 25452 2004-10-26 17:23:37Z dbinger $
"""
from sancho.utest import UTest
from dulcinea.timestamped import Timestamped, timestamp_sorted, \
     reverse_timestamp_sorted


class Test (UTest):

    def a(self):
        x = Timestamped()
        y = Timestamped()
        assert timestamp_sorted([x, y]) == [x,y]
        assert reverse_timestamp_sorted([x, y]) == [y,x]
        x.set_timestamp()
        assert timestamp_sorted([x, y]) == [y,x]

if __name__ == "__main__":
    Test()



"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/news.py $
$Id: news.py 25462 2004-10-26 19:15:15Z dbinger $
"""
from dulcinea.base import DulcineaPersistent
from dulcinea.keyed import Keyed, KeyedMap
from dulcinea.timestamped import Timestamped, reverse_timestamp_sorted


class NewsItem(Keyed, Timestamped, DulcineaPersistent):
    """
    A single announcement.

    Instance attributes:
      title : string
        Title of this announcement.
      subtitle : string
        A subtitle that can be shown with the link text, but isn't required.
      text : string
        HTML text of this announcement.  This field will not be escaped.
    """

    title_is = str
    subtitle_is = str
    text_is = str

    def __init__ (self):
        Keyed.__init__(self)
        Timestamped.__init__(self)
        self.title = ""
        self.subtitle = ""
        self.text = ""

    def get_title(self):
        return self.title

    def get_subtitle(self):
        return self.subtitle

    def get_text(self):
        return self.text

    def set_title (self, title):
        self.title = title

    def set_subtitle(self, subtitle):
        self.subtitle = subtitle

    def set_text (self, text):
        self.text = text


class NewsDatabase(KeyedMap, DulcineaPersistent):
    """
    Instance attributes: none
    """

    def __init__(self):
        KeyedMap.__init__(self, value_spec=NewsItem)

    def delete_news_item(self, item):
        """(item : NewsItem)
        """
        del self.mapping[item.get_key()]

    def get_news_item(self, key):
        return self.mapping[key]

    def get_news_items(self):
        return self.mapping.values()

    def get_sorted_news_items(self):
        return reverse_timestamp_sorted(self.mapping.values())



"""$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/__init__.py $
$Id: __init__.py 25637 2004-11-18 15:21:56Z dbinger $
"""

from dulcinea.site_util import SiteModule
local = SiteModule('local')
local_ui = SiteModule('local_ui')
del SiteModule
# local and local_ui are site-specific 'modules' that can be imported
# from dulcinea.  This allows code in dulcinea to use site-specific
# values.
# import local and local_ui from here, but do not access
# their attributes at the module level or you may have
# circular import problems.
# Don't do this:
#     from dulcinea.local import foo # Bad.
#     ...
#     foo()
# Do this:
#     from dulcinea import local # Good.
#     ...
#     local.foo()



"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/ui/directory.py $
$Id: directory.py 26276 2005-03-02 23:26:38Z rmasse $
"""
from quixote import get_request, redirect, get_path
from dulcinea.ui.errors import not_found

class DynamicExportingDirectory (object):

    def _q_translate(self, component):
        """(component : string) -> string | None
        """
        for export, name, crumb, title in self.get_exports():
            if component == export:
                return name
        return None

    def _q_lookup(self, component):
        """(component : string) -> object

        Lookup a path component and return the corresponding object (usually
        a Directory, a method or a string).  Returning None signals that the
        component does not exist.
        """
        return None

    def _q_traverse(self, path):
        """(path: [string]) -> object

        Traverse a path and return the result.
        """
        component = path[0]
        name = self._q_translate(component)
        if name is None:
            obj = self._q_lookup(component)
        else:
            obj = getattr(self, name)
        if obj is None:
            not_found('directory %r has no component %r' % (self, component))
        if path[1:]:
            if hasattr(obj, '_q_traverse'):
                result = obj._q_traverse(path[1:])
            else:
                not_found()
        elif callable(obj):
            result = obj()
        else:
            result = obj
        if result is None:
            not_found()
        else:
            return result

    def __call__(self):
        if (self._q_translate('') is not None and
            not get_request().get_fields()):
            # Fix missing trailing slash.
            return redirect(get_path() + "/", permanent=True)
        else:
            return None



DynamicExporting = DynamicExportingDirectory




